import{d as a,r as n,a as r,b as c,e as t,f as s,t as i,g as d,o as l}from"./index-xAnPWFJT.js";const _={style:{"background-color":"#d2d2fa",height:"100vh",display:"grid","place-items":"center"}},h={style:{"text-align":"center"}},p=t("a",{href:"https://github.com/xaoyaoo/PyWxDump.git"},"PyWxDump",-1),g={style:{"text-align":"center"}},y=a({__name:"HomeView",setup(m){const o=n("");return r(()=>{d().then(e=>{o.value=e}).catch(e=>{console.error("Error fetching API version:",e)})}),(e,f)=>(l(),c("div",_,[t("h2",h,[s("欢迎使用"),p,s("聊天记录查看工具! "),t("p",g,"当前版本："+i(o.value),1)])]))}});export{y as default};
