.table-success-row[data-v-a0244981]{--el-table-tr-bg-color: var(--el-color-success-light-9)}.el-tooltip__popper .popper__content[data-v-0836fbf7]{white-space:pre-line}.label_color[data-v-b3b47fb2]{color:#333;font-size:15px;padding-left:15px;padding-right:0}.data_color[data-v-b3b47fb2]{color:#08488c;background-color:#f4f4f4;font-size:15px;padding-left:6px;padding-right:6px;font-weight:700;white-space:nowrap;max-width:80%}.button_color[data-v-b3b47fb2]{color:#0048ff;font-size:15px;padding-left:15px;padding-right:0;text-decoration:underline}
